/******************************************************************************
      (C) Copyright 1992 by Autodesk, Inc.

      This program is copyrighted by Autodesk, Inc. and is  licensed
      to you under the following conditions.  You may not distribute
      or  publish the source code of this program in any form.   You
      may  incorporate this code in object form in derivative  works
      provided  such  derivative  works  are  (i.) are  designed and 
      intended  to  work  solely  with  Autodesk, Inc. products, and 
      (ii.)  contain  Autodesk's  copyright  notice  "(C)  Copyright  
      1992 by Autodesk, Inc."

      AUTODESK  PROVIDES THIS PROGRAM "AS IS" AND WITH  ALL  FAULTS.
      AUTODESK  SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTY OF  MER-
      CHANTABILITY OR FITNESS FOR A PARTICULAR USE.  AUTODESK,  INC.
      DOES  NOT  WARRANT THAT THE OPERATION OF THE PROGRAM  WILL  BE
      UNINTERRUPTED OR ERROR FREE.

*******************************************************************************/

/* For the Metaware High C and High C/C++ compilers, turn off an
   unwanted warning message */
#ifdef __HIGHC__
pragma Offwarn(67); /* kills "switch statement has no cases" warning */
#endif

#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include "sxp.h"

#define LIM255(c) (((c)>=255)?255:(c))

/* Variable definitions */
#define SIZE 1
#define WIDTH 2
#define COL1 4
#define COLM 45
#define COL2 5
#define SHINE1 6
#define SHINEM 67
#define SHINE2 7
#define FRAME1 8
#define FRAMEM 89
#define FRAME2 9
/* The above definitions are the LineID numbers for
   the dialog lines in which the variables will be set.
   The integers assigned to the dialog lines containing
   variables are selected by the IPAS programmer and
   used by 3D Studio to identify the variables. */

/************************/

/* Dialog description */
DlgEntry cdialog[]={
        0,"TITLE=\"Vary Color and Shininess (Animated)\"",
                                             /* Text to appear in dialog box */
        0,"TITLE=\"\"",                            /* Blank line for spacing */
        0,"TITLE=\"Version 2.0 by Choi Moon-Sun (aeroplus)\"",
                                             /* Text to appear in dialog box */
        0,"TITLE=\"\"",                            /* Blank line for spacing */
        COL1,"COLOR=\"Start Color\"",
                             /* Setup for standard color triple slider dialog*/
        COLM,"-COLOR=\"Mid Color\"",
                             /* Setup for standard color triple slider dialog*/
        COL2,"-COLOR=\"End Color\"",
                             /* Setup for standard color triple slider dialog*/
        0,"TITLE=\"\"",                            /* Blank line for spacing */
        SHINE1, "LINT=\"Start Shine:\",0,100,4",
                                          /* Setup for 'limited integer type
                                     variariable and associated range values */
        SHINEM, "-LINT=\"Mid Shine:\",0,100,4",  /* Limited integer variable */
        SHINE2, "-LINT=\"End Shine:\",0,100,4",  /* Limited integer variable */
        0,"TITLE=\"\"",                            /* Blank line for spacing */
        FRAME1, "LINT=\"Start Frame:\",0,32000,6",
                                                 /* Limited integer variable */
        FRAMEM, "-LINT=\"Mid Frame:\",0,32000,6",
                                                 /* Limited integer variable */
        FRAME2, "-LINT=\"End Frame:\",0,32000,6",
                                                 /* Limited integer variable */
        0,"TITLE=\"\"",                            /* Blank line for spacing */
	0,NULL
	};

/* Version test value */
/* Every external process must have a unique version number  */
#define VARY_VERS 0xF003

/* State Structure definition */
/* Any variables whose values are set via dialog interface
   must be declared within the 'State' struct. */
typedef struct {
	ulong version;
        Col24 col1,colm, col2;
        float shine1,shinem, shine2;
        long frame1,framem, frame2;
	} VaryState;

/* the "state" struct MUST start with a "ulong" which is the version#,
	to prevent using data from old versions of this program.
	This verification is performed automatically. */

#define INIT_VARY { VARY_VERS, \
                    {0.0,0.0,0.0},{115,102,128},{230,204,155},.2,.4,.8,0,15,30}

static VaryState init_state = INIT_VARY;					/* Initial state settings */
static VaryState state = INIT_VARY;					/* Default state settings */

/*----------------------------------------------------------------*/


static void lerp_color(Col24 *c, Col24 *a, Col24 *b, float f) {
	int alph,ialph;
	if (f>1.0) f = 1.0;
	alph = (int)4096*f;
	ialph = 4096-alph;
	c->r = (ialph*a->r + alph*b->r)>>12;
	c->g = (ialph*a->g + alph*b->g)>>12;
	c->b = (ialph*a->b + alph*b->b)>>12;
	}

/* The following function is a 'Client' procedure
   that defines a 3-dimensional function to modify the
   diffuse color and/or the shininess. This function
   is passed the SXPDATA structure contained in the
   EXPbuf, but has no access to packet commands or
   to other EXPbuf data structures. */

void Client3DText(Sxpdata *sd) {
    float r;
    int nframes;

    if (sd->curfield<=2*state.frame1) {
        r = 0.0;
    }
    else if (sd->curfield>=2*state.frame2) {
        r = 2.0;
    }
    else {
        nframes = state.frame2-state.frame1;
        if (nframes<=0)
            r = 0.0;
        else {
            if (sd->curfield>=2*state.framem) {
                nframes = state.frame2-state.framem;
                r = 1.0 + (float)(sd->curfield-2*state.framem) /
                          (float)(2*nframes);
            }
            else {
                nframes = state.framem-state.frame1;
                r = (float)(sd->curfield-2*state.frame1) / (float)(2*nframes);
            }
        }
    }

    if (r < 1.0) {
        sd->shininess = state.shine1+(state.shinem-state.shine1)*r;
        lerp_color(&sd->col,&state.col1,&state.colm,r);
    }
    else {
        r--;
        sd->shininess = state.shinem+(state.shine2-state.shinem)*r;
        lerp_color(&sd->col,&state.colm,&state.col2,r);
    }
}

/* The following function is a 'Client' procedure
   that defines a 3-dimensional function to compute an
   opacity value. This function is passed the SXPDATA
   structure contained in the EXPbuf, but has no access
   to packet commands or to other EXPbuf data structures. */

void Client3DOpac(Sxpdata *sd) {
    Client3DText(sd);
}

/* The following function is a 'Client' procedure
   that defines a 3-dimensional function to compute a
   normal perturbation value. This function is passed
   the SXPDATA structure contained in the EXPbuf, but
   has no access to packet commands or to other
   EXPbuf data structures. */

void Client3DBump(Sxpdata *sd) { }

/* Following function sets the appropriate 'State' variables based
   on the input from the dialog */
void ClientSetStateVar(int id, void *ptr) {
        OVL o;        /* Union of possible values that the dialog can return */
	ulong *ul;

	ul=(ulong *)ptr;
	o.ul = *ul;
	switch(id) {
		case SHINE1: 					/* State variables set here based on
								values stored in the fields of the 
								Union type .*/
			state.shine1 = ((float)o.ul)/100.0; 
			break;
        case SHINEM:
            state.shinem = ((float)o.ul)/100.0;
            break;
        case SHINE2: 
			state.shine2 = ((float)o.ul)/100.0; 
			break;
		case COL1: 					/* Set each field within the COL 
								variable in RGB format */
			state.col1.r = o.c.r;
			state.col1.g = o.c.g;
			state.col1.b = o.c.b;
			break;
        case COLM:                  /* Set each field within the COL
								variable in RGB format */
            state.colm.r = o.c.r;
            state.colm.g = o.c.g;
            state.colm.b = o.c.b;
			break;
        case COL2:                  /* Set each field within the COL
								variable in RGB format */
			state.col2.r = o.c.r;
			state.col2.g = o.c.g;
			state.col2.b = o.c.b;
            break;
        case FRAME1: state.frame1 = o.l; break;
        case FRAMEM: state.framem = o.l; break;
        case FRAME2: state.frame2 = o.l; break;
        }
	}

/* Following function sets the appropriate value of the Union 'OVL' variable
   to return values stored in the 'State' variable */
ulong ClientGetStateVar(int id) {
	OVL o;
	switch(id) {
        case SHINE1: o.ul = state.shine1*100; break;
                     /* Union variables set to appropriate value
							from 'state' variables */
        case SHINEM: o.ul = state.shinem*100; break;
        case SHINE2: o.ul = state.shine2*100; break;
		case COL1:
			o.c.r = state.col1.r;
			o.c.g = state.col1.g;
			o.c.b = state.col1.b;
			break;
        case COLM:
            o.c.r = state.colm.r;
            o.c.g = state.colm.g;
            o.c.b = state.colm.b;
            break;
        case COL2:
			o.c.r = state.col2.r;
			o.c.g = state.col2.g;
			o.c.b = state.col2.b;
			break;
		case FRAME1: o.l = state.frame1; break;
        case FRAMEM: o.l = state.framem; break;
        case FRAME2: o.l = state.frame2; break;
		}
	return(o.ul);					/* the variable requested by 3D Studio is 
							returned to 3D Studio via the 'ulong' field
							of the 'Union' structure 'OVL'. */
	}

/* ---For most applications, the following functions will not change */

/* Following function allows 3D Studio to determine the size of
   the 'State' structure and returns a pointer to the 'State' struct */

char  *ClientGetState(int *size) {
	*size = sizeof(VaryState);
	return((char *)&state);
	}

/* Following function allows 3D Studio to reset the state of the
   external process */
void ClientResetState(void) {	
	state = init_state;	
	}

/* The following function performs any calculations necessary to the operation of 3DTEXT, 3DBUMP, or 
3DOPAC that cannot be performed within any of these functions. The 'ClientStartUp' procedure does not 
have access to packet-commands. */

void ClientStartup(void) {
	/* allocate data structures, compute lookup tables, etc */
	}

void ClientTerminate(void) { 
	/* free any data structures, etc. */
	}

/* The following function allows 3D Studio to get the address of a string containing a variable. The IPAS
programmer will not have to call this function or modify the following code, but will have to include it */

DlgEntry *ClientDialog(int n) {	
	return(&cdialog[n]); 
	}

/* Following function allows 3D Studio to determine the size of
   any variable in the dialog */

int ClientVarSize(int id) {
	switch(id) {
		default:
			return(1);
		}
	}

